require File.dirname(__FILE__) + '/../../../../test_helper'
require 'test/unit'
require 'set'

include OpenEhr::RM::DataTypes::Text

class RM_Common_Generic_Test < Test::Unit::TestCase
  def setup
    assert_nothing_raised(Exception){party_proxy = PartyProxy.new}
    object_id = ObjectId.new('0.0.4')
    party_ref = PartyRef.new('unknown', 'ORGANISATION', object_id)
    assert_nothing_raised(Exception){
      @party_proxy = PartyProxy.new(:external_ref => party_ref)}
    assert_nothing_raised(Exception){party_self = PartySelf.new}
    assert_nothing_raised(Exception){
      @party_self = PartySelf.new(:external_ref => party_ref)}
    assert_raise(ArgumentError){
      party_identified = PartyIdentified.new}

    identifiers = []
    identifiers << DvIdentifier.new('NERV', 'MELCHIOR', 'GENDO', 'COMPUTER')
    identifiers << DvIdentifier.new('NERV', 'CASPER', 'GENDO', 'COMPUTER')
    identifiers << DvIdentifier.new('NERV', 'BALTHAZAR', 'GENDO', 'COMPUTER')
    assert_nothing_raised(Exception){
      @party_identified = PartyIdentified.new(:name => 'NERV',
                                               :external_ref => party_ref,
                                               :identifier => identifiers)}

    terminology_id = TerminologyId.new('test(0.04)')
    code_phrase = CodePhrase.new('self', terminology_id)
    dv_coded_text = DvCodedText.new('Seele',code_phrase)
    assert_nothing_raised(Exception){
      @party_related = PartyRelated.new(:name => 'GEHIRN',
                                         :relationship => dv_coded_text)}
    dv_text = DvText.new('commiter')
    dv_coded_text = DvCodedText.new('present',terminology_id)
    dv_date_time1 = OpenEhr::RM::DataTypes::Quantity::DateTime::DvDateTime.new('2009-07-04T16:30:00')
    dv_date_time2 = OpenEhr::RM::DataTypes::Quantity::DateTime::DvDateTime.new('2009-07-14T00:00:00')
    dv_interval = OpenEhr::RM::DataTypes::Quantity::DvInterval.new(dv_date_time1, dv_date_time2)
    assert_nothing_raised(Exception){
      @participation = Participation.new(:performer => @party_proxy,
                                         :function => dv_text,
                                         :mode => dv_coded_text,
                                         :time => dv_interval)}
    dv_date_time = OpenEhr::RM::DataTypes::Quantity::DateTime::DvDateTime.new('2009-07-04T18:56:00')
    terminology_id = TerminologyId.new('openehr(1.0.2)')
    code_phrase = CodePhrase.new('249', terminology_id)
    dv_coded_text = DvCodedText.new('creation', code_phrase)
    dv_text = DvText.new('test environment')
    assert_nothing_raised(Exception){
      @audit_details = AuditDetails.new(:system_id => 'MAGI',
                                         :committer => @party_proxy,
                                         :time_committed => dv_date_time,
                                         :change_type => dv_coded_text,
                                         :description => dv_text)}
    dv_text = DvText.new('authorisation')
    dv_ehr_uri = OpenEhr::RM::DataTypes::Uri::DvEhrUri.new('ehr://test/')
    assert_nothing_raised(Exception){
      @attestation = Attestation.new(:system_id => 'NERV',
                                     :committer => @party_proxy,
                                     :time_committed => dv_date_time,
                                     :change_type => dv_coded_text,
                                     :reason => dv_text,
                                     :description => dv_text,
                                     :proof => 'hashedstring',
                                     :items => Set[dv_ehr_uri])}
    object_version_id = ObjectVersionId.new('SHOGOUKI::NERV::1.1.1')
    audit_details2 = AuditDetails.new(:system_id => 'MAGI',
                                       :committer => @party_proxy,
                                       :time_committed => dv_date_time2,
                                       :change_type => dv_coded_text)
    assert_nothing_raised(Exception){
      @revision_history_item = RevisionHistoryItem.new(:audits => [@audit_details, audit_details2],
                                                         :version_id => object_version_id)}
    dv_date_time3 = OpenEhr::RM::DataTypes::Quantity::DateTime::DvDateTime.new('2009-07-14T12:00:00')
    dv_date_time4 = OpenEhr::RM::DataTypes::Quantity::DateTime::DvDateTime.new('2009-07-15T00:00:00')
    audit_details3 = AuditDetails.new(:system_id => 'MAGI',
                                     :committer => @party_proxy,
                                     :time_committed => dv_date_time3,
                                     :change_type => dv_coded_text)
    audit_details4 = AuditDetails.new(:system_id => 'MAGI',
                                     :committer => @party_proxy,
                                     :time_committed => dv_date_time4,
                                     :change_type => dv_coded_text)
    object_version_id = ObjectVersionId.new('NIGOUKI::NERV::2.2.2')
    revision_history_item2 = RevisionHistoryItem.new(:audits => [audit_details3, audit_details4],
                                                      :version_id => object_version_id)
    assert_nothing_raised(Exception){
      @revision_history = RevisionHistory.new([@revision_history_item, revision_history_item2])}
  end

  def test_init
    assert_instance_of PartyProxy, @party_proxy
    assert_instance_of PartySelf, @party_self
    assert_instance_of PartyIdentified, @party_identified
    assert_instance_of Participation, @participation
    assert_instance_of AuditDetails, @audit_details
    assert_instance_of Attestation, @attestation
    assert_instance_of RevisionHistoryItem, @revision_history_item
    assert_instance_of RevisionHistory, @revision_history
  end

  def test_party_proxy
    assert_equal 'unknown', @party_proxy.external_ref.namespace
  end

  def test_party_self
    assert_equal 'ORGANISATION', @party_self.external_ref.type
  end

  def test_party_identified
    assert_equal 'NERV', @party_identified.name
    assert_equal '0.0.4', @party_identified.external_ref.id.value
    identifiers = @party_identified.identifier
    ids = [ ]
    identifiers.each do |id|
      ids << id.id
    end
    assert_equal %w[MELCHIOR CASPER BALTHAZAR], ids
  end

  def test_party_related
    assert_equal 'GEHIRN', @party_related.name
  end

  def test_participation
    assert_equal 'unknown', @participation.performer.external_ref.namespace
    assert_equal 'commiter', @participation.function.value
    assert_equal 'present', @participation.mode.value
    assert_equal 2009, @participation.time.lower.year
  end

  def test_audit_details
    assert_equal 'MAGI', @audit_details.system_id
    assert_equal 'unknown', @audit_details.committer.external_ref.namespace
    assert_equal '2009-07-04T18:56:00', @audit_details.time_committed.as_string
    assert_equal 'creation', @audit_details.change_type.value
    assert_equal 'test environment', @audit_details.description.value
  end

  def test_attestation
    assert_equal 'NERV', @attestation.system_id
    assert_equal 'unknown', @attestation.committer.external_ref.namespace
    assert_equal '249', @attestation.change_type.defining_code.code_string
    assert_equal 7, @attestation.time_committed.month
    assert_equal 'authorisation', @attestation.reason.value
    assert_equal 'authorisation', @attestation.description.value
    assert_equal 'hashedstring', @attestation.proof
    assert_equal 'ehr://test/', @attestation.items.to_a[0].value
  end

  def test_revision_history_item
    assert_equal 'SHOGOUKI::NERV::1.1.1', @revision_history_item.version_id.value
    assert_equal 'MAGI', @revision_history_item.audits[0].system_id
  end

  def test_revision_history
    assert_equal 'NIGOUKI::NERV::2.2.2', @revision_history.most_recent_version
    assert_equal '2009-07-14T12:00:00', @revision_history.most_recent_version_time_committed
  end
end
